"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const validator_1 = __importDefault(require("./validator"));
const numbers_1 = __importDefault(require("../lib/numbers"));
const alphabet_1 = __importDefault(require("../lib/alphabet"));
/**
 * RingCentral password. Used to log into any RC software or the web interface. Not needed if a customer account enforces SSO.
 *
 * https://support.ringcentral.com/article/5766.html
 */
class PasswordValidator extends validator_1.default {
    constructor() {
        super();
        this.atLeastOneLetterRegex = new RegExp(/[a-zA-Z]{1,}/);
        this.atLeastOneNumberRegex = new RegExp(/\d{1,}/);
        this.atLeastOneUpperOrSpecialRegex = new RegExp(/[A-Z]{1,}|[!@#$%^&*]{1,}/);
        this.sequentialNumbersRegex = this.createSequentialNumbersRegex();
        this.repeatingNumbersRegex = this.createRepeatedNumbersRegex();
        this.sequentialLettersRegex = this.createSequentialLettersRegex();
        this.repeatingLettersRegex = this.createRepeatedLettersRegex();
    }
    ;
    toolTip() {
        return "Passwords require an upper case, lower case, number, and special character";
    }
    ;
    validate(element) {
        let errors = [];
        if (!this.atLeastOneLetterRegex.test(element)) {
            errors.push('Passwords require at least one letter');
        }
        ;
        if (!this.atLeastOneNumberRegex.test(element)) {
            errors.push('Passwords require at least one number');
        }
        ;
        if (!this.atLeastOneUpperOrSpecialRegex.test(element)) {
            errors.push('Passwords require at least one special or upper case character');
        }
        ;
        if (element.length < 8 || element.length > 32) {
            errors.push('Password length must be between 8 and 32 characters');
        }
        ;
        if (this.sequentialNumbersRegex.test(element)) {
            errors.push('Passwords cannot contain 3 sequential numbers, like 123');
        }
        ;
        if (this.repeatingNumbersRegex.test(element)) {
            errors.push('Passwords cannot contain 3 repeating numbers, like 111');
        }
        ;
        if (this.sequentialLettersRegex.test(element)) {
            errors.push('Passwords cannot contain 3 sequential letters, like abc');
        }
        ;
        if (this.repeatingLettersRegex.test(element)) {
            errors.push('Passwords cannot contain 3 repeating letters, like aaa');
        }
        ;
        return new Promise((resolve, reject) => {
            if (errors.length > 0) {
                reject({
                    originalElement: element,
                    errors: errors
                });
            }
            else {
                resolve(true);
            }
            ;
        });
    }
    ;
    createRepeatedNumbersRegex() {
        let repeatedNumArr = [];
        for (let i = 0; i < numbers_1.default.length; i++) {
            repeatedNumArr.push(numbers_1.default[i] + numbers_1.default[i] + numbers_1.default[i]);
        }
        ;
        return new RegExp("(" + repeatedNumArr.join('|') + ")", "i");
    }
    ;
    createSequentialNumbersRegex() {
        let sequentialNumbers = [];
        for (let i = 0; i < numbers_1.default.length; i++) {
            if (i == 0 || i == numbers_1.default.length - 1) {
                continue;
            }
            ;
            sequentialNumbers.push(numbers_1.default[i - 1] + numbers_1.default[i] + numbers_1.default[i + 1]);
        }
        ;
        numbers_1.default.reverse();
        for (let i = 0; i < numbers_1.default.length; i++) {
            if (i == 0 || i == numbers_1.default.length - 1) {
                continue;
            }
            ;
            sequentialNumbers.push(numbers_1.default[i - 1] + numbers_1.default[i] + numbers_1.default[i + 1]);
        }
        ;
        return new RegExp("(" + sequentialNumbers.join('|') + ")", "i");
    }
    ;
    createRepeatedLettersRegex() {
        let repeatedLettersArr = [];
        for (let i = 0; i < alphabet_1.default.length; i++) {
            repeatedLettersArr.push(alphabet_1.default[i] + alphabet_1.default[i] + alphabet_1.default[i]);
        }
        ;
        return new RegExp("(" + repeatedLettersArr.join('|') + ")", "i");
    }
    ;
    createSequentialLettersRegex() {
        let sequentialLettersArray = [];
        for (let i = 0; i < alphabet_1.default.length; i++) {
            if (i == 0 || i == alphabet_1.default.length - 1) {
                continue;
            }
            ;
            sequentialLettersArray.push(alphabet_1.default[i - 1] + alphabet_1.default[i] + alphabet_1.default[i + 1]);
        }
        ;
        numbers_1.default.reverse();
        for (let i = 0; i < alphabet_1.default.length; i++) {
            if (i == 0 || i == alphabet_1.default.length - 1) {
                continue;
            }
            ;
            sequentialLettersArray.push(alphabet_1.default[i - 1] + alphabet_1.default[i] + alphabet_1.default[i + 1]);
        }
        ;
        return new RegExp("(" + sequentialLettersArray.join('|') + ")", "i");
    }
    ;
}
;
exports.default = PasswordValidator;
